@@ -13,9 +13,9 @@ from stanza.models.common import loss
 from stanza.models.common import utils
 from stanza.models.pos.vocab import CharVocab
 
-from stanza.models.classifiers.classifier_args import WVType, ExtraVectors
-from stanza.models.classifiers.trainer import Trainer
 import stanza.models.classifiers.data as data
+from stanza.models.classifiers.trainer import Trainer
+from stanza.models.classifiers.utils import WVType, ExtraVectors, ModelType
 
 from stanza.utils.confusion import format_confusion, confusion_to_accuracy, confusion_to_macro_f1
 
@@ -24,6 +24,7 @@ class Loss(Enum):
     CROSS = 1
     WEIGHTED_CROSS = 2
     LOG_CROSS = 3
+    FOCAL = 4
 
 class DevScoring(Enum):
     ACCURACY = 'ACC'
@@ -170,7 +171,12 @@ def build_parser():
     parser.add_argument('--train_file', type=str, default=DEFAULT_TRAIN, help='Input file(s) to train a model from.  Each line is an example.  Should go <label> <tokenized sentence>.  Comma separated list.')
     parser.add_argument('--dev_file', type=str, default=DEFAULT_DEV, help='Input file(s) to use as the dev set.')
     parser.add_argument('--test_file', type=str, default=DEFAULT_TEST, help='Input file(s) to use as the test set.')
+    parser.add_argument('--output_predictions', default=False, action='store_true', help='Output predictions when running the test set')
     parser.add_argument('--max_epochs', type=int, default=100)
+    parser.add_argument('--tick', type=int, default=2000)
+
+    parser.add_argument('--model_type', type=lambda x: ModelType[x.upper()], default=ModelType.CNN,
+                        help='Model type to use.  Options: %s' % " ".join(x.name for x in ModelType))
 
     parser.add_argument('--filter_sizes', default=(3,4,5), type=ast.literal_eval, help='Filter sizes for the layer after the word vectors')
     parser.add_argument('--filter_channels', default=1000, type=ast.literal_eval, help='Number of channels for layers after the word vectors.  Int for same number of channels (scaled by width) for each filter, or tuple/list for exact lengths for each filter')
@@ -198,6 +204,7 @@ def build_parser():
 
     parser.add_argument('--loss', type=lambda x: Loss[x.upper()], default=Loss.CROSS,
                         help="Whether to use regular cross entropy or scale it by 1/log(quantity)")
+    parser.add_argument('--loss_focal_gamma', default=2, type=float, help='gamma value for a focal loss')
     parser.add_argument('--min_train_len', type=int, default=0,
                         help="Filter sentences less than this length")
 
@@ -256,13 +263,24 @@ def build_parser():
 
     parser.add_argument('--maxpool_width', type=int, default=1, help="Width of the maxpool kernel to use")
 
+    parser.add_argument('--no_constituency_backprop', dest='constituency_backprop', default=True, action='store_false', help="When using a constituency parser, backprop into the parser's weights if True")
+    parser.add_argument('--constituency_model', type=str, default="/home/john/stanza_resources/it/constituency/vit_bert.pt", help="Which constituency model to use.  TODO: make this more user friendly")
+    parser.add_argument('--constituency_batch_norm', default=False, action='store_true', help='Add a LayerNorm between the output of the parser and the classifier layers')
+    parser.add_argument('--constituency_node_attn', default=False, action='store_true', help='True means to make an attn layer out of the tree, with the words as key and nodes as query')
+    parser.add_argument('--no_constituency_node_attn', dest='constituency_node_attn', action='store_false', help='True means to make an attn layer out of the tree, with the words as key and nodes as query')
+    parser.add_argument('--constituency_top_layer', dest='constituency_top_layer', default=False, action='store_true', help='True means use the top (ROOT) layer of the constituents.  Otherwise, the next layer down (S, usually) will be used')
+    parser.add_argument('--no_constituency_top_layer', dest='constituency_top_layer', action='store_false', help='True means use the top (ROOT) layer of the constituents.  Otherwise, the next layer down (S, usually) will be used')
+    parser.add_argument('--constituency_all_words', default=False, action='store_true', help='Use all word positions in the constituency classifier')
+    parser.add_argument('--no_constituency_all_words', dest='constituency_all_words', default=False, action='store_false', help='Use the start and end word embeddings as inputs to the constituency classifier')
+
+    parser.add_argument('--log_norms', default=False, action='store_true', help='Log the parameters norms while training.  A very noisy option')
+
     parser.add_argument('--wandb', action='store_true', help='Start a wandb session and write the results of training.  Only applies to training.  Use --wandb_name instead to specify a name')
     parser.add_argument('--wandb_name', default=None, help='Name of a wandb session to start when training.  Will default to the dataset short name')
 
     parser.add_argument('--seed', default=None, type=int, help='Random seed for model')
 
-    parser.add_argument('--cuda', action='store_true', help='Use CUDA for training/testing', default=torch.cuda.is_available())
-    parser.add_argument('--cpu', action='store_false', help='Ignore CUDA.', dest='cuda')
+    utils.add_device_args(parser)
 
     return parser
 
@@ -288,7 +306,27 @@ def parse_args(args=None):
     return args
 
 
-def confusion_dataset(model, dataset):
+def dataset_predictions(model, dataset):
+    model.eval()
+    index_label_map = {x: y for (x, y) in enumerate(model.labels)}
+
+    dataset_lengths = data.sort_dataset_by_len(dataset, keep_index=True)
+
+    predictions = []
+    o_idx = []
+    for length in dataset_lengths.keys():
+        batch = dataset_lengths[length]
+        output = model([x[0] for x in batch])
+        for i in range(len(batch)):
+            predicted = torch.argmax(output[i])
+            predicted_label = index_label_map[predicted.item()]
+            predictions.append(predicted_label)
+            o_idx.append(batch[i][1])
+
+    predictions = utils.unsort(predictions, o_idx)
+    return predictions
+
+def confusion_dataset(predictions, dataset, labels):
     """
     Returns a confusion matrix
 
@@ -296,25 +334,13 @@ def confusion_dataset(model, dataset):
     Second key: predicted
     so: confusion_matrix[gold][predicted]
     """
-    model.eval()
-    index_label_map = {x: y for (x, y) in enumerate(model.labels)}
-
-    dataset_lengths = data.sort_dataset_by_len(dataset)
-
     confusion_matrix = {}
-    for label in model.labels:
+    for label in labels:
         confusion_matrix[label] = {}
 
-    for length in dataset_lengths.keys():
-        batch = dataset_lengths[length]
-        text = [x[1] for x in batch]
-        expected_labels = [x[0] for x in batch]
-
-        output = model(text)
-        for i in range(len(expected_labels)):
-            predicted = torch.argmax(output[i])
-            predicted_label = index_label_map[predicted.item()]
-            confusion_matrix[expected_labels[i]][predicted_label] = confusion_matrix[expected_labels[i]].get(predicted_label, 0) + 1
+    for predicted_label, datum in zip(predictions, dataset):
+        expected_label = datum.sentiment
+        confusion_matrix[expected_label][predicted_label] = confusion_matrix[expected_label].get(predicted_label, 0) + 1
 
     return confusion_matrix
 
@@ -341,10 +367,9 @@ def score_dataset(model, dataset, label_map=None,
     for length in dataset_lengths.keys():
         # TODO: possibly break this up into smaller batches
         batch = dataset_lengths[length]
-        text = [x[1] for x in batch]
-        expected_labels = [label_map[x[0]] for x in batch]
+        expected_labels = [label_map[x.sentiment] for x in batch]
 
-        output = model(text)
+        output = model(batch)
 
         for i in range(len(expected_labels)):
             predicted = torch.argmax(output[i])
@@ -375,7 +400,8 @@ def score_dataset(model, dataset, label_map=None,
     return correct
 
 def score_dev_set(model, dev_set, dev_eval_scoring):
-    confusion_matrix = confusion_dataset(model, dev_set)
+    predictions = dataset_predictions(model, dev_set)
+    confusion_matrix = confusion_dataset(predictions, dev_set, model.labels)
     logger.info("Dev set confusion matrix:\n{}".format(format_confusion(confusion_matrix, model.labels)))
     correct, total = confusion_to_accuracy(confusion_matrix)
     macro_f1 = confusion_to_macro_f1(confusion_matrix)
@@ -421,12 +447,24 @@ def train_model(trainer, model_file, checkpoint_file, args, train_set, dev_set,
     label_tensors = {x: torch.tensor(y, requires_grad=False, device=device)
                      for (y, x) in enumerate(labels)}
 
+    process_outputs = lambda x: x
     if args.loss == Loss.CROSS:
+        logger.info("Creating CrossEntropyLoss")
         loss_function = nn.CrossEntropyLoss()
     elif args.loss == Loss.WEIGHTED_CROSS:
+        logger.info("Creating weighted cross entropy loss w/o log")
         loss_function = loss.weighted_cross_entropy_loss([label_map[x[0]] for x in train_set], log_dampened=False)
     elif args.loss == Loss.LOG_CROSS:
+        logger.info("Creating weighted cross entropy loss w/ log")
         loss_function = loss.weighted_cross_entropy_loss([label_map[x[0]] for x in train_set], log_dampened=True)
+    elif args.loss == Loss.FOCAL:
+        try:
+            from focal_loss.focal_loss import FocalLoss
+        except ImportError:
+            raise ImportError("focal_loss not installed.  Must `pip install focal_loss_torch` to use the --loss=focal feature")
+        logger.info("Creating FocalLoss with loss %f", args.loss_focal_gamma)
+        process_outputs = lambda x: torch.softmax(x, dim=1)
+        loss_function = FocalLoss(gamma=args.loss_focal_gamma)
     else:
         raise ValueError("Unknown loss function {}".format(args.loss))
     loss_function.to(device)
@@ -457,34 +495,37 @@ def train_model(trainer, model_file, checkpoint_file, args, train_set, dev_set,
         running_loss = 0.0
         epoch_loss = 0.0
         shuffled = data.shuffle_dataset(train_set_by_len)
+
         model.train()
+        logger.info("Starting epoch %d", trainer.epochs_trained)
+        if args.log_norms:
+            model.log_norms()
+
         random.shuffle(batch_starts)
         for batch_num, start_batch in enumerate(batch_starts):
             trainer.global_step += 1
             logger.debug("Starting batch: %d step %d", start_batch, trainer.global_step)
 
             batch = shuffled[start_batch:start_batch+args.batch_size]
-            text = [x[1] for x in batch]
-            label = torch.stack([label_tensors[x[0]] for x in batch])
+            batch_labels = torch.stack([label_tensors[x.sentiment] for x in batch])
 
             # zero the parameter gradients
             optimizer.zero_grad()
 
-            outputs = model(text)
-            batch_loss = loss_function(outputs, label)
+            outputs = model(batch)
+            outputs = process_outputs(outputs)
+            batch_loss = loss_function(outputs, batch_labels)
             batch_loss.backward()
             optimizer.step()
 
             # print statistics
             running_loss += batch_loss.item()
-            if ((batch_num + 1) * args.batch_size) % 2000 < args.batch_size: # print every 2000 items
-                train_loss = running_loss / 2000
-                logger.info('[%d, %5d] Average loss: %.3f' %
-                            (trainer.epochs_trained + 1, ((batch_num + 1) * args.batch_size), train_loss))
+            if ((batch_num + 1) * args.batch_size) % args.tick < args.batch_size: # print every 2000 items
+                train_loss = running_loss / args.tick
+                logger.info('[%d, %5d] Average loss: %.3f', trainer.epochs_trained + 1, (batch_num + 1) * args.batch_size, train_loss)
                 if args.wandb:
                     wandb.log({'train_loss': train_loss}, step=trainer.global_step)
-                if (args.dev_eval_steps > 0 and
-                    ((batch_num + 1) * args.batch_size) % args.dev_eval_steps < args.batch_size):
+                if args.dev_eval_steps > 0 and ((batch_num + 1) * args.batch_size) % args.dev_eval_steps < args.batch_size:
                     logger.info('---- Interim analysis ----')
                     dev_score, accuracy, macro_f1 = score_dev_set(model, dev_set, args.dev_eval_scoring)
                     if args.wandb:
@@ -516,18 +557,9 @@ def train_model(trainer, model_file, checkpoint_file, args, train_set, dev_set,
     if args.wandb:
         wandb.finish()
 
-def print_args(args):
-    """
-    For record keeping purposes, print out the arguments when training
-    """
-    args = vars(args)
-    keys = sorted(args.keys())
-    log_lines = ['%s: %s' % (k, args[k]) for k in keys]
-    logger.info('ARGS USED AT TRAINING TIME:\n%s\n' % '\n'.join(log_lines))
-
 def main(args=None):
     args = parse_args(args)
-    seed = utils.set_random_seed(args.seed, args.cuda)
+    seed = utils.set_random_seed(args.seed)
     logger.info("Using random seed: %d" % seed)
 
     utils.ensure_dir(args.save_dir)
@@ -567,14 +599,12 @@ def main(args=None):
     else:
         trainer = Trainer.build_new_model(args, train_set)
 
-    logger.info("Filter sizes: %s" % str(trainer.model.config.filter_sizes))
-    logger.info("Filter channels: %s" % str(trainer.model.config.filter_channels))
-    logger.info("Intermediate layers: %s" % str(trainer.model.config.fc_shapes))
+    trainer.model.log_configuration()
 
     model_file = os.path.join(args.save_dir, save_name)
 
     if args.train:
-        print_args(args)
+        utils.log_training_args(args, logger)
 
         dev_set = data.read_dataset(args.dev_file, args.wordvec_type, min_len=None)
         logger.info("Using dev set: %s", args.dev_file)
@@ -584,12 +614,17 @@ def main(args=None):
 
         train_model(trainer, model_file, checkpoint_file, args, train_set, dev_set, trainer.model.labels)
 
+    if args.log_norms:
+        trainer.model.log_norms()
     test_set = data.read_dataset(args.test_file, args.wordvec_type, min_len=None)
     logger.info("Using test set: %s" % args.test_file)
     data.check_labels(trainer.model.labels, test_set)
 
     if args.test_remap_labels is None:
-        confusion_matrix = confusion_dataset(trainer.model, test_set)
+        predictions = dataset_predictions(trainer.model, test_set)
+        confusion_matrix = confusion_dataset(predictions, test_set, trainer.model.labels)
+        if args.output_predictions:
+            logger.info("List of predictions: %s", predictions)
         logger.info("Confusion matrix:\n{}".format(format_confusion(confusion_matrix, trainer.model.labels)))
         correct, total = confusion_to_accuracy(confusion_matrix)
         logger.info("Macro f1: {}".format(confusion_to_macro_f1(confusion_matrix)))
