@@ -10,12 +10,13 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
-import stanza.models.classifiers.classifier_args as classifier_args
 import stanza.models.classifiers.data as data
+from stanza.models.classifiers.base_classifier import BaseClassifier
+from stanza.models.classifiers.data import SentimentDatum
+from stanza.models.classifiers.utils import ExtraVectors, ModelType, build_output_layers
 from stanza.models.common.bert_embedding import extract_bert_embeddings
 from stanza.models.common.data import get_long_tensor, sort_all
 from stanza.models.common.foundation_cache import load_bert
-from stanza.models.common.utils import split_into_batches, sort_with_indices, unsort
 from stanza.models.common.vocab import PAD_ID, UNK_ID
 
 """
@@ -50,7 +51,7 @@ help, but dev performance went down for each variation of
 logger = logging.getLogger('stanza')
 tlogger = logging.getLogger('stanza.classifiers.trainer')
 
-class CNNClassifier(nn.Module):
+class CNNClassifier(BaseClassifier):
     def __init__(self, pretrain, extra_vocab, labels,
                  charmodel_forward, charmodel_backward, elmo_model, bert_model, bert_tokenizer,
                  args):
@@ -88,7 +89,7 @@ class CNNClassifier(nn.Module):
                                       bilstm = args.bilstm,
                                       bilstm_hidden_dim = args.bilstm_hidden_dim,
                                       maxpool_width = args.maxpool_width,
-                                      model_type = 'CNNClassifier')
+                                      model_type = ModelType.CNN)
 
         self.char_lowercase = args.char_lowercase
 
@@ -123,12 +124,12 @@ class CNNClassifier(nn.Module):
         # replacing NBSP picks up a whole bunch of words for VI
         self.vocab_map = { word.replace('\xa0', ' '): i for i, word in enumerate(pretrain.vocab) }
 
-        if self.config.extra_wordvec_method is not classifier_args.ExtraVectors.NONE:
+        if self.config.extra_wordvec_method is not ExtraVectors.NONE:
             if not extra_vocab:
                 raise ValueError("Should have had extra_vocab set for extra_wordvec_method {}".format(self.config.extra_wordvec_method))
             if not args.extra_wordvec_dim:
                 self.config.extra_wordvec_dim = self.embedding_dim
-            if self.config.extra_wordvec_method is classifier_args.ExtraVectors.SUM:
+            if self.config.extra_wordvec_method is ExtraVectors.SUM:
                 if self.config.extra_wordvec_dim != self.embedding_dim:
                     raise ValueError("extra_wordvec_dim must equal embedding_dim for {}".format(self.config.extra_wordvec_method))
 
@@ -150,11 +151,11 @@ class CNNClassifier(nn.Module):
 
         # Pytorch is "aware" of the existence of the nn.Modules inside
         # an nn.ModuleList in terms of parameters() etc
-        if self.config.extra_wordvec_method is classifier_args.ExtraVectors.NONE:
+        if self.config.extra_wordvec_method is ExtraVectors.NONE:
             total_embedding_dim = self.embedding_dim
-        elif self.config.extra_wordvec_method is classifier_args.ExtraVectors.SUM:
+        elif self.config.extra_wordvec_method is ExtraVectors.SUM:
             total_embedding_dim = self.embedding_dim
-        elif self.config.extra_wordvec_method is classifier_args.ExtraVectors.CONCAT:
+        elif self.config.extra_wordvec_method is ExtraVectors.CONCAT:
             total_embedding_dim = self.embedding_dim + self.config.extra_wordvec_dim
         else:
             raise ValueError("unable to handle {}".format(self.config.extra_wordvec_method))
@@ -240,13 +241,7 @@ class CNNClassifier(nn.Module):
                 raise ValueError("Expected int or 2d tuple for conv size")
 
         tlogger.debug("Input dim to FC layers: %d", self.fc_input_size)
-        fc_layers = []
-        previous_layer_size = self.fc_input_size
-        for shape in self.config.fc_shapes:
-            fc_layers.append(nn.Linear(previous_layer_size, shape))
-            previous_layer_size = shape
-        fc_layers.append(nn.Linear(previous_layer_size, self.config.num_classes))
-        self.fc_layers = nn.ModuleList(fc_layers)
+        self.fc_layers = build_output_layers(self.fc_input_size, self.config.fc_shapes, self.config.num_classes)
 
         self.dropout = nn.Dropout(self.config.dropout)
 
@@ -254,6 +249,24 @@ class CNNClassifier(nn.Module):
         self.unsaved_modules += [name]
         setattr(self, name, module)
 
+    def is_unsaved_module(self, name):
+        return name.split('.')[0] in self.unsaved_modules
+
+    def log_configuration(self):
+        """
+        Log some essential information about the model configuration to the training logger
+        """
+        tlogger.info("Filter sizes: %s" % str(self.config.filter_sizes))
+        tlogger.info("Filter channels: %s" % str(self.config.filter_channels))
+        tlogger.info("Intermediate layers: %s" % str(self.config.fc_shapes))
+
+    def log_norms(self):
+        lines = ["NORMS FOR MODEL PARAMTERS"]
+        for name, param in self.named_parameters():
+            if param.requires_grad and name.split(".")[0] not in ('bert_model', 'forward_charlm', 'backward_charlm'):
+                lines.append("%s %.6g" % (name, torch.norm(param).item()))
+        logger.info("\n".join(lines))
+
     def build_char_reps(self, inputs, max_phrase_len, charlm, projection, begin_paddings, device):
         char_reps = charlm.build_char_representation(inputs)
         if projection is not None:
@@ -289,6 +302,7 @@ class CNNClassifier(nn.Module):
                     return idx
             return vocab_map.get(word.lower(), UNK_ID)
 
+        inputs = [x.text if isinstance(x, SentimentDatum) else x for x in inputs]
         # we will pad each phrase so either it matches the longest
         # conv or the longest phrase in the input, whichever is longer
         max_phrase_len = max(len(x) for x in inputs)
@@ -379,9 +393,9 @@ class CNNClassifier(nn.Module):
         if self.extra_vocab:
             extra_batch_indices = torch.tensor(extra_batch_indices, requires_grad=False, device=device)
             extra_input_vectors = self.extra_embedding(extra_batch_indices)
-            if self.config.extra_wordvec_method is classifier_args.ExtraVectors.CONCAT:
+            if self.config.extra_wordvec_method is ExtraVectors.CONCAT:
                 all_inputs = [input_vectors, extra_input_vectors]
-            elif self.config.extra_wordvec_method is classifier_args.ExtraVectors.SUM:
+            elif self.config.extra_wordvec_method is ExtraVectors.SUM:
                 all_inputs = [input_vectors + extra_input_vectors]
             else:
                 raise ValueError("unable to handle {}".format(self.config.extra_wordvec_method))
@@ -448,36 +462,26 @@ class CNNClassifier(nn.Module):
         # https://discuss.pytorch.org/t/multi-class-cross-entropy-loss-and-softmax-in-pytorch/24920/4
         return out
 
+    def get_params(self, skip_modules=True):
+        model_state = self.state_dict()
+        # skip saving modules like pretrained embeddings, because they are large and will be saved in a separate file
+        if skip_modules:
+            skipped = [k for k in model_state.keys() if self.is_unsaved_module(k)]
+            for k in skipped:
+                del model_state[k]
 
-def label_text(model, text, batch_size=None):
-    """
-    Given a list of sentences, return the model's results on that text.
-    """
-    model.eval()
+        params = {
+            'model':        model_state,
+            'config':       self.config,
+            'labels':       self.labels,
+            'extra_vocab':  self.extra_vocab,
+        }
+        return params
 
-    text = [data.update_text(s, model.config.wordvec_type) for s in text]
+    def preprocess_data(self, sentences):
+        sentences = [data.update_text(s, self.config.wordvec_type) for s in sentences]
+        return sentences
 
-    if batch_size is None:
-        intervals = [(0, len(text))]
-        orig_idx = None
-    else:
-        text, orig_idx = sort_with_indices(text, key=len, reverse=True)
-        intervals = split_into_batches(text, batch_size)
-    labels = []
-    for interval in intervals:
-        if interval[1] - interval[0] == 0:
-            # this can happen for empty text
-            continue
-        output = model(text[interval[0]:interval[1]])
-        predicted = torch.argmax(output, dim=1)
-        labels.extend(predicted.tolist())
-
-    if orig_idx:
-        text = unsort(text, orig_idx)
-        labels = unsort(labels, orig_idx)
-
-    logger.debug("Found labels")
-    for (label, sentence) in zip(labels, text):
-        logger.debug((label, sentence))
-
-    return labels
+    def extract_sentences(self, doc):
+        # TODO: tokens or words better here?
+        return [[token.text for token in sentence.tokens] for sentence in doc.sentences]
