@@ -19,47 +19,39 @@ from stanza.models.lemma.vocab import MultiVocab
 
 logger = logging.getLogger('stanza')
 
-def unpack_batch(batch, use_cuda):
+def unpack_batch(batch, device):
     """ Unpack a batch from the data loader. """
-    if use_cuda:
-        inputs = [b.cuda() if b is not None else None for b in batch[:6]]
-    else:
-        inputs = [b if b is not None else None for b in batch[:6]]
+    inputs = [b.to(device) if b is not None else None for b in batch[:6]]
     orig_idx = batch[6]
     return inputs, orig_idx
 
 class Trainer(object):
     """ A trainer for training models. """
-    def __init__(self, args=None, vocab=None, emb_matrix=None, model_file=None, use_cuda=False):
-        self.use_cuda = use_cuda
+    def __init__(self, args=None, vocab=None, emb_matrix=None, model_file=None, device=None):
         if model_file is not None:
             # load everything from file
-            self.load(model_file, use_cuda)
+            self.load(model_file)
         else:
             # build model from scratch
             self.args = args
-            self.model = None if args['dict_only'] else Seq2SeqModel(args, emb_matrix=emb_matrix, use_cuda=use_cuda)
+            self.model = None if args['dict_only'] else Seq2SeqModel(args, emb_matrix=emb_matrix)
             self.vocab = vocab
             # dict-based components
             self.word_dict = dict()
             self.composite_dict = dict()
         if not self.args['dict_only']:
+            self.model = self.model.to(device)
             if self.args.get('edit', False):
-                self.crit = loss.MixLoss(self.vocab['char'].size, self.args['alpha'])
+                self.crit = loss.MixLoss(self.vocab['char'].size, self.args['alpha']).to(device)
                 logger.debug("Running seq2seq lemmatizer with edit classifier...")
             else:
-                self.crit = loss.SequenceLoss(self.vocab['char'].size)
+                self.crit = loss.SequenceLoss(self.vocab['char'].size).to(device)
             self.parameters = [p for p in self.model.parameters() if p.requires_grad]
-            if use_cuda:
-                self.model.cuda()
-                self.crit.cuda()
-            else:
-                self.model.cpu()
-                self.crit.cpu()
             self.optimizer = utils.get_optimizer(self.args['optim'], self.parameters, self.args['lr'])
 
     def update(self, batch, eval=False):
-        inputs, orig_idx = unpack_batch(batch, self.use_cuda)
+        device = next(self.model.parameters()).device
+        inputs, orig_idx = unpack_batch(batch, device)
         src, src_mask, tgt_in, tgt_out, pos, edits = inputs
 
         if eval:
@@ -84,7 +76,8 @@ class Trainer(object):
         return loss_val
 
     def predict(self, batch, beam_size=1):
-        inputs, orig_idx = unpack_batch(batch, self.use_cuda)
+        device = next(self.model.parameters()).device
+        inputs, orig_idx = unpack_batch(batch, device)
         src, src_mask, tgt, tgt_mask, pos, edits = inputs
 
         self.model.eval()
@@ -195,7 +188,7 @@ class Trainer(object):
         torch.save(params, filename, _use_new_zipfile_serialization=False)
         logger.info("Model saved to {}".format(filename))
 
-    def load(self, filename, use_cuda=False):
+    def load(self, filename):
         try:
             checkpoint = torch.load(filename, lambda storage, loc: storage)
         except BaseException:
@@ -204,8 +197,10 @@ class Trainer(object):
         self.args = checkpoint['config']
         self.word_dict, self.composite_dict = checkpoint['dicts']
         if not self.args['dict_only']:
-            self.model = Seq2SeqModel(self.args, use_cuda=use_cuda)
-            self.model.load_state_dict(checkpoint['model'])
+            self.model = Seq2SeqModel(self.args)
+            # could remove strict=False after rebuilding all models,
+            # or could switch to 1.6.0 torch with the buffer in seq2seq persistent=False
+            self.model.load_state_dict(checkpoint['model'], strict=False)
         else:
             self.model = None
         self.vocab = MultiVocab.load_state_dict(checkpoint['vocab'])
