@@ -96,8 +96,7 @@ def parse_args(args=None):
     parser.add_argument('--save_name', type=str, default=None, help="File name to save the model")
 
     parser.add_argument('--seed', type=int, default=1234)
-    parser.add_argument('--cuda', type=bool, default=torch.cuda.is_available())
-    parser.add_argument('--cpu', action='store_true', help='Ignore CUDA.')
+    utils.add_device_args(parser)
 
     parser.add_argument('--wandb', action='store_true', help='Start a wandb session and write the results of training.  Only applies to training.  Use --wandb_name instead to specify a name')
     parser.add_argument('--wandb_name', default=None, help='Name of a wandb session to start when training.  Will default to the dataset short name')
@@ -106,8 +105,6 @@ def parse_args(args=None):
 
     if args.wandb_name:
         args.wandb = True
-    if args.cpu:
-        args.cuda = False
 
     args = vars(args)
     return args
@@ -115,12 +112,12 @@ def parse_args(args=None):
 def main(args=None):
     args = parse_args(args=args)
 
-    utils.set_random_seed(args['seed'], args['cuda'])
+    utils.set_random_seed(args['seed'])
 
     logger.info("Running NER tagger in {} mode".format(args['mode']))
 
     if args['mode'] == 'train':
-        train(args)
+        return train(args)
     else:
         evaluate(args)
 
@@ -138,10 +135,20 @@ def load_pretrain(args):
         pretrain = Pretrain(None, vec_file, args['pretrain_max_vocab'], save_to_file=False)
     return pretrain
 
+# TODO: refactor with the same thing in tagger.py and elsewhere
+def model_file_name(args):
+    if args['save_name'] is not None:
+        save_name = args['save_name']
+    else:
+        save_name = args['shorthand'] + "_nertagger.pt"
+
+    if not os.path.exists(os.path.join(args['save_dir'], save_name)) and os.path.exists(save_name):
+        return save_name
+    return os.path.join(args['save_dir'], save_name)
+
 def train(args):
-    utils.ensure_dir(args['save_dir'])
-    model_file = os.path.join(args['save_dir'], args['save_name']) if args['save_name'] \
-        else '{}/{}_nertagger.pt'.format(args['save_dir'], args['shorthand'])
+    model_file = model_file_name(args)
+    utils.ensure_dir(os.path.split(model_file)[0])
 
     pretrain = None
     vocab = None
@@ -205,7 +212,7 @@ def train(args):
 
     logger.info("Training tagger...")
     if trainer is None: # init if model was not loaded previously from file
-        trainer = Trainer(args=args, vocab=vocab, pretrain=pretrain, use_cuda=args['cuda'],
+        trainer = Trainer(args=args, vocab=vocab, pretrain=pretrain, device=args['device'],
                           train_classifier_only=args['train_classifier_only'])
     logger.info(trainer.model)
 
@@ -219,6 +226,8 @@ def train(args):
 
     # LR scheduling
     if args['lr_decay'] > 0:
+        # learning rate changes on plateau -- no improvement on model for patience number of epochs
+        # change is made as a factor of the learning rate decay
         scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(trainer.optimizer, mode='max', factor=args['lr_decay'], \
             patience=args['patience'], verbose=True, min_lr=args['min_lr'])
     else:
@@ -296,6 +305,8 @@ def train(args):
         logger.info("Dev set never evaluated.  Saving final model.")
         trainer.save(model_file)
 
+    return trainer
+
 def write_ner_results(filename, batch, preds):
     if len(batch.tags) != len(preds):
         raise ValueError("Unexpected batch vs pred lengths: %d vs %d" % (len(batch.tags), len(preds)))
@@ -316,8 +327,7 @@ def write_ner_results(filename, batch, preds):
 
 def evaluate(args):
     # file paths
-    model_file = os.path.join(args['save_dir'], args['save_name']) if args['save_name'] \
-        else '{}/{}_nertagger.pt'.format(args['save_dir'], args['shorthand'])
+    model_file = model_file_name(args)
 
     loaded_args, trainer, vocab = load_model(args, model_file)
     logger.debug("Loaded model for eval from %s", model_file)
@@ -349,14 +359,13 @@ def evaluate(args):
 
 def load_model(args, model_file):
     # load model
-    use_cuda = args['cuda'] and not args['cpu']
     charlm_args = {}
     if 'charlm_forward_file' in args:
         charlm_args['charlm_forward_file'] = args['charlm_forward_file']
     if 'charlm_backward_file' in args:
         charlm_args['charlm_backward_file'] = args['charlm_backward_file']
     pretrain = load_pretrain(args)
-    trainer = Trainer(args=charlm_args, model_file=model_file, pretrain=pretrain, use_cuda=use_cuda, train_classifier_only=args['train_classifier_only'])
+    trainer = Trainer(args=charlm_args, model_file=model_file, pretrain=pretrain, device=args['device'], train_classifier_only=args['train_classifier_only'])
     loaded_args, vocab = trainer.args, trainer.vocab
 
     # load config
