@@ -1,18 +1,11 @@
-"""Processor that attaches a constituency tree to a sentence
-
-The model used is a generally a model trained on the Stanford
-Sentiment Treebank or some similar dataset.  When run, this processor
-attaches a score in the form of a string to each sentence in the
-document.
-
-TODO: a possible way to generalize this would be to make it a
-ClassifierProcessor and have "sentiment" be an option.
+"""
+Processor that attaches a constituency tree to a sentence
 """
 
-import stanza.models.constituency.trainer as trainer
+from stanza.models.constituency.trainer import Trainer
 
 from stanza.models.common import doc
-from stanza.models.common.utils import get_tqdm
+from stanza.utils.get_tqdm import get_tqdm
 from stanza.pipeline._constants import *
 from stanza.pipeline.processor import UDProcessor, register_processor
 
@@ -35,7 +28,7 @@ class ConstituencyProcessor(UDProcessor):
         else:
             self._requires = self.__class__.REQUIRES_DEFAULT
 
-    def _set_up_model(self, config, pipeline, use_gpu):
+    def _set_up_model(self, config, pipeline, device):
         # set up model
         # pretrain and charlm paths are args from the config
         # bert (if used) will be chosen from the model save file
@@ -43,12 +36,13 @@ class ConstituencyProcessor(UDProcessor):
             "wordvec_pretrain_file": config.get('pretrain_path', None),
             "charlm_forward_file": config.get('forward_charlm_path', None),
             "charlm_backward_file": config.get('backward_charlm_path', None),
-            "cuda": use_gpu,
+            "device": device,
         }
-        self._model = trainer.Trainer.load(filename=config['model_path'],
-                                           args=args,
-                                           foundation_cache=pipeline.foundation_cache)
-        self._model.model.eval()
+        trainer = Trainer.load(filename=config['model_path'],
+                               args=args,
+                               foundation_cache=pipeline.foundation_cache)
+        self._model = trainer.model
+        self._model.eval()
         # batch size counted as sentences
         self._batch_size = int(config.get('batch_size', ConstituencyProcessor.DEFAULT_BATCH_SIZE))
         self._tqdm = 'tqdm' in config and config['tqdm']
@@ -63,7 +57,7 @@ class ConstituencyProcessor(UDProcessor):
         if self._tqdm:
             words = tqdm(words)
 
-        trees = trainer.parse_tagged_words(self._model.model, words, self._batch_size)
+        trees = self._model.parse_tagged_words(words, self._batch_size)
         document.set(CONSTITUENCY, trees, to_sentence=True)
         return document
 
@@ -74,4 +68,4 @@ class ConstituencyProcessor(UDProcessor):
         For a pipeline, this can be queried with
           pipeline.processors["constituency"].get_constituents()
         """
-        return set(self._model.model.constituents)
+        return set(self._model.constituents)
