@@ -32,12 +32,12 @@ class TokenizeProcessor(UDProcessor):
     # default max sequence length
     MAX_SEQ_LENGTH_DEFAULT = 1000
 
-    def _set_up_model(self, config, pipeline, use_gpu):
+    def _set_up_model(self, config, pipeline, device):
         # set up trainer
         if config.get('pretokenized'):
             self._trainer = None
         else:
-            self._trainer = Trainer(model_file=config['model_path'], use_cuda=use_gpu)
+            self._trainer = Trainer(model_file=config['model_path'], device=device)
 
     def process_pre_tokenized_text(self, input_src):
         """
@@ -65,8 +65,8 @@ class TokenizeProcessor(UDProcessor):
         return raw_text, document
 
     def process(self, document):
-        assert isinstance(document, str) or isinstance(document, doc.Document) or (self.config.get('pretokenized') or self.config.get('no_ssplit', False)), \
-            "If neither 'pretokenized' or 'no_ssplit' option is enabled, the input to the TokenizerProcessor must be a string or a Document object."
+        if not (isinstance(document, str) or isinstance(document, doc.Document) or (self.config.get('pretokenized') or self.config.get('no_ssplit', False))):
+            raise ValueError("If neither 'pretokenized' or 'no_ssplit' option is enabled, the input to the TokenizerProcessor must be a string or a Document object.  Got %s" % str(type(document)))
 
         if isinstance(document, doc.Document):
             if self.config.get('pretokenized'):
@@ -81,14 +81,24 @@ class TokenizeProcessor(UDProcessor):
             return self._variant.process(document)
 
         raw_text = '\n\n'.join(document) if isinstance(document, list) else document
+
+        max_seq_len = self.config.get('max_seqlen', TokenizeProcessor.MAX_SEQ_LENGTH_DEFAULT)
+
         # set up batches
         batches = TokenizationDataset(self.config, input_text=raw_text, vocab=self.vocab, evaluation=True, dictionary=self.trainer.dictionary)
         # get dict data
         _, _, _, document = output_predictions(None, self.trainer, batches, self.vocab, None,
-                                               self.config.get('max_seqlen', TokenizeProcessor.MAX_SEQ_LENGTH_DEFAULT),
+                                               max_seq_len,
                                                orig_text=raw_text,
                                                no_ssplit=self.config.get('no_ssplit', False),
                                                num_workers = self.config.get('num_workers', 0))
+
+        # replace excessively long tokens with <UNK> to avoid downstream GPU memory issues in POS
+        for sentence in document:
+            for token in sentence:
+                if len(token['text']) > max_seq_len:
+                    token['text'] = "<UNK>"
+
         return doc.Document(document, raw_text)
 
     def bulk_process(self, docs):
