@@ -5,7 +5,8 @@ Also produces the resources.json file.
 
 For example, on the cluster, you can do this:
 
-python3 -m stanza.resources.prepare_resources --input_dir /u/nlp/software/stanza/models/current-models --output_dir /u/nlp/software/stanza/models/1.4.1 > resources.out 2>&1
+python3 -m stanza.resources.prepare_resources --input_dir /u/nlp/software/stanza/models/current-models-1.5.0 --output_dir /u/nlp/software/stanza/models/1.5.0 > resources.out 2>&1
+nlprun -q john "python3 -m stanza.resources.prepare_resources --input_dir /u/nlp/software/stanza/models/current-models-1.5.0 --output_dir /u/nlp/software/stanza/models/1.5.0" -o resources.out
 """
 
 import json
@@ -20,8 +21,8 @@ from stanza.models.common.constant import lcode2lang, two_to_three_letters
 
 def parse_args():
     parser = argparse.ArgumentParser()
-    parser.add_argument('--input_dir', type=str, default="/u/nlp/software/stanza/current-models", help='Input dir for various models.  Defaults to the recommended home on the nlp cluster')
-    parser.add_argument('--output_dir', type=str, default="/u/nlp/software/stanza/built-models", help='Output dir for various models.')
+    parser.add_argument('--input_dir', type=str, default="/u/nlp/software/stanza/current-models-1.5.0", help='Input dir for various models.  Defaults to the recommended home on the nlp cluster')
+    parser.add_argument('--output_dir', type=str, default="/u/nlp/software/stanza/1.5.0", help='Output dir for various models.')
     args = parser.parse_args()
     args.input_dir = os.path.abspath(args.input_dir)
     args.output_dir = os.path.abspath(args.output_dir)
@@ -73,11 +74,13 @@ default_treebanks = {
     "fro":     "srcmf",
     "fa":      "perdt",
     "my":      "ucsy",
+    "myv":     "jr",
     "pl":      "pdb",
     "pt":      "bosque",
     "ro":      "rrt",
     "ru":      "syntagrus",
     "sa":      "vedic",
+    "sd":      "isra",
     "sr":      "set",
     "sk":      "snk",
     "sl":      "ssj",
@@ -122,6 +125,7 @@ no_pretrain_languages = set([
 default_pretrains = dict(default_treebanks)
 for lang in no_pretrain_languages:
     default_pretrains.pop(lang, None)
+default_pretrains["myv"] = "mokha"
 
 # default ner for languages
 default_ners = {
@@ -136,6 +140,7 @@ default_ners = {
     "fi": "turku",
     "fr": "wikiner",
     "hu": "combined",
+    "hy": "armtdp",
     "it": "fbk",
     "ja": "gsd",
     "kk": "kazNERD",
@@ -146,6 +151,7 @@ default_ners = {
     "nn": "norne",
     "pl": "nkjp",
     "ru": "wikiner",
+    "sd": "siner",
     "sv": "suc3shuffle",
     "th": "lst20",
     "tr": "starlang",
@@ -166,6 +172,8 @@ default_charlms = {
     "fa": "conll17",
     "fi": "conll17",
     "fr": "newswiki",
+    "he": "oscar",
+    "hi": "oscar",
     "it": "conll17",
     "ja": "conll17",
     "kk": "oscar",
@@ -175,7 +183,9 @@ default_charlms = {
     "nl": "ccwiki",
     "pl": "oscar",
     "ru": "newswiki",
+    "sd": "isra",
     "sv": "conll17",
+    "te": "oscar2022",
     "th": "oscar",
     "tr": "conll17",
     "uk": "conll17",
@@ -195,6 +205,13 @@ pos_charlms = {
     },
 }
 
+# TODO: retrain all of the depparse models with the charlm
+depparse_charlms = {
+    "en": {
+        "combined": "1billion",
+    }
+}
+
 ner_charlms = {
     "en": {
         "conll03": "1billion",
@@ -218,6 +235,29 @@ ner_charlms = {
     },
 }
 
+# TODO: eventually we want to
+#   - rename all the pretrains to indicate where they are from
+#   - only have special / unique names for the few which need it, such as the bio pretrains
+pos_pretrains = {
+    "en": {
+        "combined_bert": "combined",
+    },
+    "it": {
+        "combined_electra": "combined",
+        "vit_bert": "vit",
+    },
+    "myv": {
+        "jr": "mokha",
+    },
+    "vi": {
+        "vlsp22_phobert_large": "vtb",
+        "vtb_phobert_base": "vtb",
+        "vtb_phobert_large": "vtb",
+    },
+}
+
+depparse_pretrains = pos_pretrains
+
 ner_pretrains = {
     "ar": {
         "aqmar": "fasttextwiki",
@@ -238,6 +278,9 @@ ner_pretrains = {
         "s800":         "craft",
 
         "ontonotes":    "fasttextcrawl",
+        # the stanza-train sample NER model should use the default NER pretrain
+        # for English, that is the same as ontonotes
+        "sample":       "fasttextcrawl",
 
         "conll03":      "glove",
 
@@ -298,6 +341,8 @@ allowed_empty_languages = [
     "th",
     # only tokenize and NER for Myanmar right now (soon...)
     "my",
+    # currently only tokenize and NER for SD as well
+    "sd",
 ]
 
 # map processor name to file ending
@@ -369,8 +414,10 @@ def get_pos_dependencies(lang, package):
     # pretrains we have floating around
     if lang in no_pretrain_languages:
         dependencies = []
-    else:
+    elif lang not in pos_pretrains or package not in pos_pretrains[lang]:
         dependencies = [{'model': 'pretrain', 'package': package}]
+    else:
+        dependencies = [{'model': 'pretrain', 'package': pos_pretrains[lang][package]}]
 
     if lang in pos_charlms and package in pos_charlms[lang]:
         charlm_package = pos_charlms[lang][package]
@@ -383,6 +430,28 @@ def get_pos_dependencies(lang, package):
 
     return dependencies
 
+def get_depparse_dependencies(lang, package):
+    if lang in no_pretrain_languages:
+        dependencies = []
+    elif lang not in depparse_pretrains or package not in depparse_pretrains[lang]:
+        dependencies = [{'model': 'pretrain', 'package': package}]
+    else:
+        dependencies = [{'model': 'pretrain', 'package': depparse_pretrains[lang][package]}]
+
+    if lang in depparse_charlms and package in depparse_charlms[lang]:
+        charlm_package = depparse_charlms[lang][package]
+    else:
+        charlm_package = None
+        # TODO: when all depparse models are retrained,
+        # use the default charlms here
+        # charlm_package = default_charlms.get(lang, None)
+
+    if charlm_package is not None:
+        dependencies.append({'model': 'forward_charlm', 'package': charlm_package})
+        dependencies.append({'model': 'backward_charlm', 'package': charlm_package})
+
+    return dependencies
+
 def get_ner_dependencies(lang, package):
     dependencies = []
 
@@ -394,7 +463,7 @@ def get_ner_dependencies(lang, package):
         dependencies = [{'model': 'pretrain', 'package': pretrain_package}]
 
     if lang not in ner_charlms or package not in ner_charlms[lang]:
-        charlm_package = default_charlms[lang]
+        charlm_package = default_charlms.get(lang, None)
     else:
         charlm_package = ner_charlms[lang][package]
 
@@ -443,8 +512,7 @@ def process_dirs(args):
             # maintain dependencies
             dependencies = None
             if processor == 'depparse':
-                if lang not in no_pretrain_languages:
-                    dependencies = [{'model': 'pretrain', 'package': package}]
+                dependencies = get_depparse_dependencies(lang, package)
             elif processor == 'pos':
                 dependencies = get_pos_dependencies(lang, package)
             elif processor == 'ner':
@@ -467,10 +535,15 @@ def process_dirs(args):
 def process_defaults(args):
     resources = json.load(open(os.path.join(args.output_dir, 'resources.json')))
     for lang in resources:
+        if all(k in ("backward_charlm", "forward_charlm", "pretrain") for k in resources[lang].keys()):
+            print(f'Skipping empty resources for language {lang}')
+            continue
         if lang not in default_treebanks: 
             raise AssertionError(f'{lang} not in default treebanks!!!')
         print(f'Preparing default models for language {lang}')
 
+        pretrains_needed = set()
+
         ud_package = default_treebanks[lang]
         os.chdir(os.path.join(args.output_dir, lang))
         default_processors = {}
@@ -478,24 +551,32 @@ def process_defaults(args):
             default_dependencies = {}
         else:
             default_dependencies = {'pos': get_pos_dependencies(lang, ud_package),
-                                    'depparse': [{'model': 'pretrain', 'package': ud_package}]}
+                                    'depparse': get_depparse_dependencies(lang, ud_package)}
+            pretrains_needed.update([dep['package'] for dep in default_dependencies['pos'] if dep['model'] == 'pretrain'])
+            pretrains_needed.update([dep['package'] for dep in default_dependencies['depparse'] if dep['model'] == 'pretrain'])
 
         if lang in default_ners:
             ner_package = default_ners[lang]
-        if lang in default_charlms:
-            charlm_package = default_charlms[lang]
-        if lang in default_ners and lang in default_charlms:
             ner_dependencies = get_ner_dependencies(lang, ner_package)
             if ner_dependencies is not None:
                 default_dependencies['ner'] = ner_dependencies
+                pretrains_needed.update([dep['package'] for dep in ner_dependencies if dep['model'] == 'pretrain'])
+        if lang in default_charlms:
+            charlm_package = default_charlms[lang]
         if lang in default_sentiment:
             sentiment_package = default_sentiment[lang]
             sentiment_dependencies = get_sentiment_dependencies(lang, package)
             default_dependencies['sentiment'] = sentiment_dependencies
+            pretrains_needed.update([dep['package'] for dep in sentiment_dependencies if dep['model'] == 'pretrain'])
         if lang in default_constituency:
             constituency_package = default_constituency[lang]
-            default_dependencies['constituency'] = get_con_dependencies(lang, constituency_package)
+            constituency_dependencies = get_con_dependencies(lang, constituency_package)
+            default_dependencies['constituency'] = constituency_dependencies
+            pretrains_needed.update([dep['package'] for dep in constituency_dependencies if dep['model'] == 'pretrain'])
 
+        # pretrain doesn't really need to be here, but by putting it here,
+        # we preserve any existing default.zip files with no other changes
+        # when rebuilding the resources
         processors = ['tokenize', 'mwt', 'lemma', 'pos', 'depparse', 'pretrain']
         if lang in default_ners:
             processors.append('ner')
@@ -512,6 +593,18 @@ def process_defaults(args):
 
         with zipfile.ZipFile('default.zip', 'w', zipfile.ZIP_DEFLATED) as zipf:
             for processor in processors:
+                if processor == 'pretrain':
+                    for package in sorted(pretrains_needed):
+                        filename = os.path.join(args.output_dir, lang, processor, package + '.pt')
+                        if os.path.exists(filename):
+                            print("   Model {} package {}: file {}".format(processor, package, filename))
+                            zipf.write(os.path.join(processor, package + '.pt'))
+                        else:
+                            raise FileNotFoundError(f"Pretrain package {package} needed for {lang} but cannot be found at {filename}")
+
+                    # done specifically with pretrains
+                    continue
+
                 if processor == 'ner': package = ner_package
                 elif processor in ['forward_charlm', 'backward_charlm']: package = charlm_package
                 elif processor == 'sentiment': package = sentiment_package
@@ -535,12 +628,13 @@ def process_defaults(args):
                     # there might be a better way to encode that here
                     default_processors[processor] = "identity"
                     print(" --Model {} package {}: no file {}, assuming identity lemmatizer".format(processor, package, filename))
-                elif processor in ('mwt', 'pretrain'):
+                elif processor == 'mwt':
                     # some languages don't have MWT, so skip ig
                     # others have pos and depparse built with no pretrain
                     print(" --Model {} package {}: no file {}, skipping".format(processor, package, filename))
                 else:
                     raise FileNotFoundError(f"Could not find an expected model file for {lang} {processor} {package} : {filename}")
+
         default_md5 = get_md5(os.path.join(args.output_dir, lang, 'default.zip'))
         resources[lang]['default_processors'] = default_processors
         resources[lang]['default_dependencies'] = default_dependencies
