@@ -116,6 +116,9 @@ def add_token(token_list, word, token):
     Add a token to a proto request.
 
     CoreNLP tokens have components of both word and token from stanza.
+
+    We pass along "after" but not "before", and the "after" is limited
+    to whether or not there is a space after the token
     """
     query_token = token_list.add()
     query_token.word = word.text
@@ -126,8 +129,29 @@ def add_token(token_list, word, token):
         query_token.pos = word.xpos
     if word.upos is not None:
         query_token.coarseTag = word.upos
+    if word.feats and word.feats != "_":
+        for feature in word.feats.split("|"):
+            key, value = feature.split("=", maxsplit=1)
+            query_token.conllUFeatures.key.append(key)
+            query_token.conllUFeatures.value.append(value)
     if token.ner is not None:
         query_token.ner = token.ner
+    if len(token.id) > 1:
+        query_token.mwtText = token.text
+        query_token.isMWT = True
+        query_token.isFirstMWT = token.id[0] == word.id
+    if token.id[-1] != word.id:
+        # if we are not the last word of an MWT token
+        # we are absolutely not followed by space
+        pass
+    elif ((token.misc is not None and "SpaceAfter=No" in token.misc) or
+          (word.misc is not None and "SpaceAfter=No" in word.misc)):
+        # if either the token or the specific word item say there is
+        # no SpaceAfter, we believe them
+        pass
+    else:
+        # otherwise, seems like there might actually be a SpaceAfter
+        query_token.after = " "
 
 def add_sentence(request_sentences, sentence, num_tokens):
     """
@@ -155,6 +179,13 @@ def add_word_to_graph(graph, word, sent_idx, word_idx):
         edge.target = word_idx+1
         edge.dep = word.deprel
 
+def features_to_string(features):
+    if not features:
+        return None
+    if len(features.key) == 0:
+        return None
+    return "|".join("%s=%s" % (key, value) for key, value in zip(features.key, features.value))
+
 class JavaProtobufContext(object):
     """
     A generic context for sending requests to a java program using protobufs in a subprocess
@@ -167,6 +198,7 @@ class JavaProtobufContext(object):
         if extra_args is None:
             extra_args = []
         self.extra_args = extra_args
+        self.pipe = None
 
     def open_pipe(self):
         self.pipe = subprocess.Popen(["java", "-cp", self.classpath, self.java_main, "-multiple"] + self.extra_args,
@@ -177,6 +209,7 @@ class JavaProtobufContext(object):
         if self.pipe.poll() is None:
             self.pipe.stdin.write((0).to_bytes(4, 'big'))
             self.pipe.stdin.flush()
+            self.pipe = None
 
     def __enter__(self):
         self.open_pipe()
@@ -186,13 +219,16 @@ class JavaProtobufContext(object):
         self.close_pipe()
 
     def process_request(self, request):
+        if self.pipe is None:
+            raise RuntimeError("Pipe to java process is not open or was closed")
+
         text = request.SerializeToString()
         self.pipe.stdin.write(len(text).to_bytes(4, 'big'))
         self.pipe.stdin.write(text)
         self.pipe.stdin.flush()
         response_length = self.pipe.stdout.read(4)
         if len(response_length) < 4:
-            raise RuntimeError("Could not communicate with java process!")
+            raise BrokenPipeError("Could not communicate with java process!")
         response_length = int.from_bytes(response_length, "big")
         response_text = self.pipe.stdout.read(response_length)
         response = self.build_response()
