@@ -65,9 +65,14 @@ Unzip the contents into a folder
 $CONSTITUENCY_BASE/italian/it_vit
 so there should be a file
 $CONSTITUENCY_BASE/italian/it_vit/VITwritten/VITconstsyntNumb
+
+There are a few other updates needed to improve the annotations,
+but all the nagging seemed to give Prof. Delmonte a headache,
+so at this point we include those fixes in this script instead.
+See the first few tsurgeon operations in update_mwts_and_special_cases
 """
 
-from collections import defaultdict, deque
+from collections import defaultdict, deque, namedtuple
 import itertools
 import os
 import re
@@ -115,6 +120,8 @@ DATE_RE = re.compile("^([0-9]{1,2})[_]([0-9]{2})$")
 PERCENT_RE = re.compile(r"^([0-9]{1,2})[_]([0-9]{1,2}[%])$")
 DECIMAL_RE = re.compile(r"^([0-9])[_]([0-9])$")
 
+ProcessedTree = namedtuple('ProcessedTree', ['con_id', 'dep_id', 'tree'])
+
 def raw_tree(text):
     """
     A sentence will look like this:
@@ -582,10 +589,10 @@ def extract_updated_dataset(con_tree_map, dep_sentence_map, split_ids, mwt_map,
         dep_sentence = dep_sentence_map[dep_id]
         updated_tree = update_tree(original_tree, dep_sentence, con_id, dep_id, mwt_map, tsurgeon_processor)
 
-        trees.append(updated_tree)
+        trees.append(ProcessedTree(con_id, dep_id, updated_tree))
     return trees
 
-def convert_it_vit(con_directory, ud_directory, output_directory, dataset_name, debug_sentence=None):
+def read_updated_trees(paths, debug_sentence=None):
     # original version with more errors
     #con_filename = os.path.join(con_directory, "2011-12-20", "Archive", "VIT_newconstsynt.txt")
     # this is the April 2022 version
@@ -593,6 +600,11 @@ def convert_it_vit(con_directory, ud_directory, output_directory, dataset_name,
     # the most recent update from ELRA may look like this?
     # it's what we got, at least
     # con_filename = os.path.join(con_directory, "italian", "VITwritten", "VITconstsyntNumb")
+
+    # needs at least UD 2.11 or this will not work
+    con_directory = paths["CONSTITUENCY_BASE"]
+    ud_directory = os.path.join(paths["UDBASE"], "UD_Italian-VIT")
+
     con_filename = os.path.join(con_directory, "italian", "it_vit", "VITwritten", "VITconstsyntNumb")
     ud_vit_train = os.path.join(ud_directory, "it_vit-ud-train.conllu")
     ud_vit_dev   = os.path.join(ud_directory, "it_vit-ud-dev.conllu")
@@ -642,7 +654,9 @@ def convert_it_vit(con_directory, ud_directory, output_directory, dataset_name,
     train_ids = match_sentences(con_tree_map, con_vit_ngrams, ud_train_data.sentences, "train", debug_sentence)
     dev_ids   = match_sentences(con_tree_map, con_vit_ngrams, ud_dev_data.sentences,   "dev",   debug_sentence)
     test_ids  = match_sentences(con_tree_map, con_vit_ngrams, ud_test_data.sentences,  "test",  debug_sentence)
-    print("Trees: {} train {} dev {} test".format(len(train_ids), len(dev_ids), len(test_ids)))
+    print("Remaining total trees: %d" % (len(train_ids) + len(dev_ids) + len(test_ids)))
+    print("  {} train {} dev {} test".format(len(train_ids), len(dev_ids), len(test_ids)))
+    print("Updating trees with MWT and newer tokens from UD...")
 
     # the moveprune feature requires a new corenlp release after 4.4.0
     with tsurgeon.Tsurgeon(classpath="$CLASSPATH") as tsurgeon_processor:
@@ -650,19 +664,28 @@ def convert_it_vit(con_directory, ud_directory, output_directory, dataset_name,
         dev_trees   = extract_updated_dataset(con_tree_map, ud_vit_dev_map,   dev_ids,   mwt_map, tsurgeon_processor)
         test_trees  = extract_updated_dataset(con_tree_map, ud_vit_test_map,  test_ids,  mwt_map, tsurgeon_processor)
 
+    return train_trees, dev_trees, test_trees
+
+def convert_it_vit(paths, dataset_name, debug_sentence=None):
+    """
+    Read the trees, then write them out to the expected output_directory
+    """
+    train_trees, dev_trees, test_trees = read_updated_trees(paths, debug_sentence)
+
+    train_trees = [x.tree for x in train_trees]
+    dev_trees   = [x.tree for x in dev_trees]
+    test_trees  = [x.tree for x in test_trees]
+
+    output_directory = paths["CONSTITUENCY_DATA_DIR"]
     write_dataset([train_trees, dev_trees, test_trees], output_directory, dataset_name)
 
 def main():
     paths = default_paths.get_default_paths()
-    con_directory = paths["CONSTITUENCY_BASE"]
-    ud_directory  = os.path.join(paths["UDBASE"], "UD_Italian-VIT")
-
-    output_directory = paths["CONSTITUENCY_DATA_DIR"]
     dataset_name = "it_vit"
 
     debug_sentence = sys.argv[1] if len(sys.argv) > 1 else None
 
-    convert_it_vit(con_directory, ud_directory, output_directory, dataset_name, debug_sentence)
+    convert_it_vit(paths, dataset_name, debug_sentence)
 
 if __name__ == '__main__':
     main()
