@@ -313,10 +313,44 @@ Masakhane NER is a set of NER datasets for African languages
     the 2 letter code for lcode.  The tool will throw an error
     if the language is not supported in Masakhane.
 
+SiNER is a Sindhi NER dataset
+  - https://aclanthology.org/2020.lrec-1.361/
+    SiNER: A Large Dataset for Sindhi Named Entity Recognition
+    Wazir Ali, Junyu Lu, Zenglin Xu
+  - It is available via git repository
+    https://github.com/AliWazir/SiNER-dataset
+    As of Nov. 2022, there were a few changes to the dataset
+    to update a couple instances of broken tags & tokenization
+  - Clone the repo to $NERBASE/sindhi
+    mkdir $NERBASE/sindhi
+    cd $NERBASE/sindhi
+    git clone git@github.com:AliWazir/SiNER-dataset.git
+  - Then, prepare the dataset with this script:
+    python3 -m stanza.utils.datasets.ner.prepare_ner_dataset sd_siner
+
+en_foreign-4class is an English non-US newswire dataset
+  - currently WIP.  annotated by MLTwist, collected at Stanford
+  - the 4 class version is converted to the 4 classes in conll,
+    then split into train/dev/test
+  - clone https://github.com/stanfordnlp/en-foreign-newswire
+    into $NERBASE/en_foreign
+
 en_sample is the toy dataset included with stanza-train
   https://github.com/stanfordnlp/stanza-train
   this is not meant for any kind of actual NER use
 
+ArmTDP-NER is an Armenian NER dataset
+  - https://github.com/myavrum/ArmTDP-NER.git
+    ArmTDP-NER: The corpus was developed by the ArmTDP team led by Marat M. Yavrumyan
+    at the Yerevan State University by the collaboration of "Armenia National SDG Innovation Lab"
+    and "UC Berkley's Armenian Linguists' network".
+  - in $NERBASE, make a "armenian" directory, then git clone the repo there
+    mkdir -p $NERBASE/armenian
+    cd $NERBASE/armenian
+    git clone https://github.com/myavrum/ArmTDP-NER.git
+  - Then run
+    python3 -m stanza.utils.datasets.ner.prepare_ner_dataset hy_armtdp
+
 """
 
 import glob
@@ -347,9 +381,12 @@ import stanza.utils.datasets.ner.convert_nytk as convert_nytk
 import stanza.utils.datasets.ner.convert_starlang_ner as convert_starlang_ner
 import stanza.utils.datasets.ner.convert_nkjp as convert_nkjp
 import stanza.utils.datasets.ner.prepare_ner_file as prepare_ner_file
+import stanza.utils.datasets.ner.convert_sindhi_siner as convert_sindhi_siner
+import stanza.utils.datasets.ner.simplify_en_foreign as simplify_en_foreign
 import stanza.utils.datasets.ner.suc_to_iob as suc_to_iob
 import stanza.utils.datasets.ner.suc_conll_to_iob as suc_conll_to_iob
-from stanza.utils.datasets.ner.utils import convert_bio_to_json, get_tags, read_tsv, write_dataset
+import stanza.utils.datasets.ner.convert_hy_armtdp as convert_hy_armtdp
+from stanza.utils.datasets.ner.utils import convert_bio_to_json, get_tags, read_tsv, write_dataset, random_shuffle_files
 
 SHARDS = ('train', 'dev', 'test')
 
@@ -936,6 +973,36 @@ def process_masakhane(paths, dataset_name):
         raise UnknownDatasetError(dataset_name, "Found the Masakhane repo, but there was no %s in the repo at path %s" % (dataset_name, in_directory))
     convert_bio_to_json(in_directory, paths["NER_DATA_DIR"], "%s_masakhane" % lcode, "txt")
 
+def process_sd_siner(paths, short_name):
+    in_directory = os.path.join(paths["NERBASE"], "sindhi", "SiNER-dataset")
+    if not os.path.exists(in_directory):
+        raise FileNotFoundError("Cannot find SiNER checkout in $NERBASE/sindhi  Please git clone to repo in that directory")
+    in_filename = os.path.join(in_directory, "SiNER-dataset.txt")
+    if not os.path.exists(in_filename):
+        in_filename = os.path.join(in_directory, "SiNER dataset.txt")
+        if not os.path.exists(in_filename):
+            raise FileNotFoundError("Found an SiNER directory at %s but the directory did not contain the dataset" % in_directory)
+    convert_sindhi_siner.convert_sindhi_siner(in_filename, paths["NER_DATA_DIR"], short_name)
+
+def process_en_foreign_4class(paths, short_name):
+    # TODO: simplify to a temp directory?
+    simplify_en_foreign.main(args=[])
+    in_directory = os.path.join(paths["NERBASE"], "en_foreign", "4class")
+    out_directory = paths["NER_DATA_DIR"]
+    random_shuffle_files(in_directory, out_directory, short_name)
+
+def process_armtdp(paths, short_name):
+    assert short_name == 'hy_armtdp'
+    base_input_path = os.path.join(paths["NERBASE"], "armenian", "ArmTDP-NER")
+    base_output_path = paths["NER_DATA_DIR"]
+    convert_hy_armtdp.convert_dataset(base_input_path, base_output_path, short_name)
+    for shard in SHARDS:
+        input_filename = os.path.join(base_output_path, f'{short_name}.{shard}.tsv')
+        if not os.path.exists(input_filename):
+            raise FileNotFoundError('Cannot find %s component of %s in %s' % (shard, short_name, input_filename))
+        output_filename = os.path.join(base_output_path, '%s.%s.json' % (short_name, shard))
+        prepare_ner_file.process_dataset(input_filename, output_filename)
+
 def process_toy_dataset(paths, short_name):
     convert_bio_to_json(os.path.join(paths["NERBASE"], "English-SAMPLE"), paths["NER_DATA_DIR"], short_name)
 
@@ -943,6 +1010,7 @@ DATASET_MAPPING = {
     "bn_daffodil":       process_bn_daffodil,
     "da_ddt":            process_da_ddt,
     "de_germeval2014":   process_de_germeval2014,
+    "en_foreign-4class": process_en_foreign_4class,
     "fa_arman":          process_fa_arman,
     "fi_turku":          process_turku,
     "hi_hiner":          process_hiner,
@@ -950,12 +1018,14 @@ DATASET_MAPPING = {
     "hi_ijc":            process_ijc,
     "hu_nytk":           process_nytk,
     "hu_combined":       process_hu_combined,
+    "hy_armtdp":         process_armtdp,
     "it_fbk":            process_it_fbk,
     "ja_gsd":            process_ja_gsd,
     "kk_kazNERD":        process_kk_kazNERD,
     "mr_l3cube":         process_mr_l3cube,
     "my_ucsy":           process_my_ucsy,
     "pl_nkjp":           process_pl_nkjp,
+    "sd_siner":          process_sd_siner,
     "sv_suc3licensed":   process_sv_suc3licensed,
     "sv_suc3shuffle":    process_sv_suc3shuffle,
     "tr_starlang":       process_starlang,
